//========================================================================================
//
//	File:				FWColor.h
//	Release Version:	$ 1.0d1 $
//
//	Creation Date:		3/28/94
//
//	Copyright:	 1993, 1994 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#ifndef FWCOLOR_H
#define FWCOLOR_H

// ----- Foundation Includes -----

#ifndef FWSTDDEF_H
#include <FWStdDef.h>
#endif

#ifndef FWSTREAM_H
#include <FWStream.h>
#endif

// ----- Macintosh Includes -----

#if defined(FW_BUILD_MAC) && !defined(__QUICKDRAW__)
#include <QuickDraw.h>
#endif

// ----- Windows Includes -----

#if defined(FW_BUILD_WIN) && !defined(_INC_WINDOWS)
#include <windows.h>
#endif

//========================================================================================
//	 Forward class declarations
//========================================================================================

//class FW_CReadableStream;
//class FW_CWritableStream;

//========================================================================================
//	 Defines
//========================================================================================

// I only define colors that are common

#ifdef FW_BUILD_WIN 
const COLORREF	FW_kRGBBlack		= RGB(0, 0, 0)
const COLORREF	FW_kRGBWhite		= RGB(255, 255, 255);
const COLORREF	FW_kRGBRed			= RGB(255, 0, 0);
const COLORREF	FW_kRGBYellow		= RGB(255, 255, 0);
const COLORREF	FW_kRGBBlue			= RGB(0, 0, 192);
const COLORREF	FW_kRGBGreen		= RGB(0, 255, 0);
const COLORREF	FW_kRGBBrown		= RGB(128, 128, 0);
const COLORREF	FW_kRGBPurple		= RGB(255, 0, 255);
const COLORREF	FW_kRGBLightBlue	= RGB(0, 0, 255);
const COLORREF	FW_kRGBGray			= RGB(128, 128, 128);
const COLORREF	FW_kRGBLightGray	= RGB(192, 192, 192);
const COLORREF	FW_kRGBDarkGray		= RGB(64, 64, 64);
#endif

#ifdef FW_BUILD_MAC
const RGBColor FW_kRGBBlack			= {0x0000, 0x0000, 0x0000};
const RGBColor FW_kRGBWhite			= {0xFFFF, 0xFFFF, 0xFFFF};
const RGBColor FW_kRGBRed			= {0xDDDD, 0x0000, 0x0000};
const RGBColor FW_kRGBYellow		= {0xFFFF, 0xFFFF, 0x0000};
const RGBColor FW_kRGBBlue			= {0x0000, 0x0000, 0xDDDD};
const RGBColor FW_kRGBGreen			= {0x0000, 0xEEEE, 0x0000};
const RGBColor FW_kRGBBrown			= {0x9999, 0x6666, 0x0000};
const RGBColor FW_kRGBPurple		= {0x6666, 0x0000, 0x9999};
const RGBColor FW_kRGBLightBlue		= {0x0000, 0x9999, 0xFFFF};
const RGBColor FW_kRGBGray			= {0x7777, 0x7777, 0x7777};
const RGBColor FW_kRGBLightGray		= {0xBBBB, 0xBBBB, 0xBBBB};
const RGBColor FW_kRGBDarkGray		= {0x5555, 0x5555, 0x5555};
#endif

//========================================================================================
//	 CLASS FW_CColor
//========================================================================================

#ifdef FW_BUILD_WIN
class FW_CColor
#endif
#ifdef FW_BUILD_MAC
class FW_CColor : public RGBColor
#endif
{
//----------------------------------------------------------------------------------------
//	 Constructors/Destructors
//
public:
	FW_CColor();
	FW_CColor(const FW_CColor& color);
	FW_CColor(unsigned short r, unsigned short g, unsigned short b);

#ifdef FW_BUILD_WIN
	FW_CColor(COLORREF rgb);
#endif

#ifdef FW_BUILD_MAC
	FW_CColor(const RGBColor* rgb);
	FW_CColor(const RGBColor& rgb);
#endif

//----------------------------------------------------------------------------------------
//	 New API
//
public:
	// ----- Comparison operators -----
	FW_Boolean operator==(const FW_CColor& clr) const;
#ifdef FW_BUILD_WIN	
	FW_Boolean operator==(COLORREF rgb) const;
#endif
#ifdef FW_BUILD_MAC	
	FW_Boolean operator==(const RGBColor& rgb) const;
#endif
	
	FW_Boolean operator!=(const FW_CColor& clr) const;
#ifdef FW_BUILD_WIN	
	FW_Boolean operator!=(COLORREF rgb) const;
#endif
#ifdef FW_BUILD_MAC	
	FW_Boolean operator!=(const RGBColor& rgb) const;
#endif

	// ----- Assignment operator -----	
	FW_CColor& operator=(const FW_CColor& color);
#ifdef FW_BUILD_WIN
	FW_CColor& operator=(COLORREF rgb);
#endif

#ifdef FW_BUILD_WIN
	operator COLORREF() const;
#endif

	// ----- Set/Get RGB Component -----
	void		SetRGB(unsigned short r, unsigned short g, unsigned short b);
	void		GetRGB(unsigned short& r, unsigned short& g, unsigned short& b) const;
	
	// ----- Arithmetic, by another color -----

	FW_CColor operator+(const FW_CColor& color) const;
	FW_CColor operator-(const FW_CColor& color) const;
	FW_CColor operator*(const FW_CColor& color) const;
	FW_CColor operator/(const FW_CColor& color) const;

	FW_CColor& operator+=(const FW_CColor& color);
	FW_CColor& operator-=(const FW_CColor& color);
	FW_CColor& operator*=(const FW_CColor& color);
	FW_CColor& operator/=(const FW_CColor& color);

	// ----- Arithmetic by a scalar -----

	FW_CColor operator+(unsigned short a) const;
	FW_CColor operator-(unsigned short a) const;
	FW_CColor operator*(unsigned short a) const;
	FW_CColor operator/(unsigned short a) const;

	FW_CColor& operator+=(unsigned short a);
	FW_CColor& operator-=(unsigned short a);
	FW_CColor& operator*=(unsigned short a);
	FW_CColor& operator/=(unsigned short a);

	void Grayer();

	FW_Boolean IsDarkerThan(const FW_CColor& color) const;
	FW_Boolean IsLighterThan(const FW_CColor& color) const;

	// ----- Stream -----
	void 		Unflatten(FW_CReadableStream& stream);
	void		Flatten(FW_CWritableStream& stream);

//----------------------------------------------------------------------------------------
//	 Private API
//
private:
	unsigned long 	Brightness() const;

	unsigned short	AddComponents(unsigned short a, unsigned short b);
	unsigned short	SubtractComponents(unsigned short a, unsigned short b);
	unsigned short	MultiplyComponents(unsigned short a, unsigned short b);
	unsigned short	DivideComponents(unsigned short a, unsigned short b);

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	 Data Members
//
private:
	COLORREF	fRGBColor;
#endif
};


//========================================================================================
//	 inlines
//========================================================================================

//----------------------------------------------------------------------------------------
//	 FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator==(const FW_CColor& color) const
{
	return (red == color.red) && 
			(green == color.green) &&
			(blue == color.blue);
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	 FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator==(COLORREF rgb) const
{
	return fRGBColor == rgb;
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	 FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator==(const RGBColor& rgb) const
{
	return (red == rgb.red) && 
			(green == rgb.green) &&
			(blue == rgb.blue);
}
#endif

//----------------------------------------------------------------------------------------
//	 FW_CColor::operator!=
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator!=(const FW_CColor& color) const
{
	return !(*this == color);
}

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	 FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator!=(COLORREF rgb) const
{
	return !(*this == rgb);
}
#endif

#ifdef FW_BUILD_MAC
//----------------------------------------------------------------------------------------
//	 FW_CColor::operator==
//----------------------------------------------------------------------------------------
inline FW_Boolean FW_CColor::operator!=(const RGBColor& rgb) const
{
	return !(*this == rgb);
}
#endif

#ifdef FW_BUILD_WIN
//----------------------------------------------------------------------------------------
//	 FW_CColor::operator COLORREF
//----------------------------------------------------------------------------------------
inline FW_CColor::operator COLORREF() const
{
	return fRGBColor;
}
#endif

#endif


